@@ -1,9 +1,10 @@
 import datetime
 
 from django.db import models
+from django.conf import settings
 from django.db.models import Q
 from django.core.cache import cache
-from django.contrib.auth.models import User
+
 from django.utils.translation import ugettext_lazy as _
 
 from friendship.signals import friendship_request_created, \
@@ -12,6 +13,7 @@ from friendship.signals import friendship_request_created, \
         friendship_removed, follower_created, following_created, follower_removed,\
         following_removed
 
+AUTH_USER_MODEL = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')
 
 CACHE_TYPES = {
     'friends': 'f-%d',
@@ -56,8 +58,8 @@ def bust_cache(type, user_pk):
 
 class FriendshipRequest(models.Model):
     """ Model to represent friendship requests """
-    from_user = models.ForeignKey(User, related_name='friendship_requests_sent')
-    to_user = models.ForeignKey(User, related_name='friendship_requests_received')
+    from_user = models.ForeignKey(AUTH_USER_MODEL, related_name='friendship_requests_sent')
+    to_user = models.ForeignKey(AUTH_USER_MODEL, related_name='friendship_requests_received')
 
     message = models.TextField(_('Message'), blank=True)
 
@@ -90,6 +92,9 @@ class FriendshipRequest(models.Model):
             )
 
         self.delete()
+        bust_cache('requests', self.to_user.pk)
+        bust_cache('friends', self.to_user.pk)
+        bust_cache('friends', self.from_user.pk)
         return True
 
     def reject(self):
@@ -97,17 +102,20 @@ class FriendshipRequest(models.Model):
         self.rejected = datetime.datetime.now()
         friendship_request_rejected.send(sender=self)
         self.save()
+        bust_cache('requests', self.to_user.pk)
 
     def cancel(self):
         """ cancel this friendship request """
         self.delete()
         friendship_request_canceled.send(sender=self)
+        bust_cache('requests', self.to_user.pk)
         return True
 
     def mark_viewed(self):
         self.viewed = datetime.datetime.now()
         friendship_request_viewed.send(sender=self)
         self.save()
+        bust_cache('requests', self.to_user.pk)
         return True
 
 
@@ -119,7 +127,7 @@ class FriendshipManager(models.Manager):
         key = cache_key('friends', user.pk)
         friends = cache.get(key)
 
-        if not friends:
+        if friends is None:
             qs = Friend.objects.select_related(depth=1).filter(to_user=user).all()
             friends = [u.from_user for u in qs]
             cache.set(key, friends)
@@ -131,7 +139,7 @@ class FriendshipManager(models.Manager):
         key = cache_key('requests', user.pk)
         requests = cache.get(key)
 
-        if not requests:
+        if requests is None:
             qs = FriendshipRequest.objects.select_related(depth=1).filter(
                     to_user=user).all()
             requests = list(qs)
@@ -144,7 +152,7 @@ class FriendshipManager(models.Manager):
         key = cache_key('unread_requests', user.pk)
         unread_requests = cache.get(key)
 
-        if not unread_requests:
+        if unread_requests is None:
             qs = FriendshipRequest.objects.select_related(depth=1).filter(
                     to_user=user,
                     viewed__isnull=True).all()
@@ -158,7 +166,7 @@ class FriendshipManager(models.Manager):
         key = cache_key('unread_request_count', user.pk)
         count = cache.get(key)
 
-        if not count:
+        if count is None:
             count = FriendshipRequest.objects.select_related(depth=1).filter(
                     to_user=user,
                     viewed__isnull=True).count()
@@ -171,7 +179,7 @@ class FriendshipManager(models.Manager):
         key = cache_key('read_requests', user.pk)
         read_requests = cache.get(key)
 
-        if not read_requests:
+        if read_requests is None:
             qs = FriendshipRequest.objects.select_related(depth=1).filter(
                     to_user=user,
                     viewed__isnull=False).all()
@@ -185,7 +193,7 @@ class FriendshipManager(models.Manager):
         key = cache_key('rejected_requests', user.pk)
         rejected_requests = cache.get(key)
 
-        if not rejected_requests:
+        if rejected_requests is None:
             qs = FriendshipRequest.objects.select_related(depth=1).filter(
                     to_user=user,
                     rejected__isnull=False).all()
@@ -201,7 +209,7 @@ class FriendshipManager(models.Manager):
                     to_user=to_user
                 )
         bust_cache('requests', to_user.pk)
-        friendship_request_created.send(sender=self)
+        friendship_request_created.send(sender=request)
 
         return request
 
@@ -245,8 +253,8 @@ class FriendshipManager(models.Manager):
 
 class Friend(models.Model):
     """ Model to represent Friendships """
-    to_user = models.ForeignKey(User, related_name='friends')
-    from_user = models.ForeignKey(User, related_name='_unused_friend_relation')
+    to_user = models.ForeignKey(AUTH_USER_MODEL, related_name='friends')
+    from_user = models.ForeignKey(AUTH_USER_MODEL, related_name='_unused_friend_relation')
     created = models.DateTimeField(default=datetime.datetime.now)
 
     objects = FriendshipManager()
@@ -268,7 +276,7 @@ class FollowingManager(models.Manager):
         key = cache_key('followers', user.pk)
         followers = cache.get(key)
 
-        if not followers:
+        if followers is None:
             qs = Follow.objects.filter(followee=user).all()
             followers = [u.follower for u in qs]
             cache.set(key, followers)
@@ -280,7 +288,7 @@ class FollowingManager(models.Manager):
         key = cache_key('following', user.pk)
         following = cache.get(key)
 
-        if not following:
+        if following is None:
             qs = Follow.objects.filter(follower=user).all()
             following = [u.followee for u in qs]
             cache.set(key, following)
@@ -331,8 +339,8 @@ class FollowingManager(models.Manager):
 
 class Follow(models.Model):
     """ Model to represent Following relationships """
-    follower = models.ForeignKey(User, related_name='following')
-    followee = models.ForeignKey(User, related_name='followers')
+    follower = models.ForeignKey(AUTH_USER_MODEL, related_name='following')
+    followee = models.ForeignKey(AUTH_USER_MODEL, related_name='followers')
     created = models.DateTimeField(default=datetime.datetime.now)
 
     objects = FollowingManager()
